@@ -4,7 +4,7 @@ use warnings;
 use strict;
 use 5.008003;
 
-our $VERSION = '1.200_01';
+our $VERSION = '1.205_01';
 use Exporter 'import';
 our @EXPORT_OK = qw( choose );
 
@@ -15,9 +15,9 @@ use Unicode::GCString;
 use Term::Choose::Constants qw( :choose );
 
 no warnings 'utf8';
-#use warnings FATAL => qw( all );
 #use Log::Log4perl qw( get_logger );
-#my $log = get_logger( 'Term::Choose' );
+# #Log::Log4perl::init() called in main::
+#my $log = get_logger( __PACKAGE__ );
 
 my $Plugin_Package;
 
@@ -81,10 +81,8 @@ sub __undef_to_defaults {
 }
 
 
-sub __validate_and_add_options {
-    my ( $self, $opt ) = @_;
-    return if ! defined $opt;
-    my $valid = {
+sub __valid_options {
+    return {
         beep            => '[ 0 1 ]',
         clear_screen    => '[ 0 1 ]',
         default         => '[ 0-9 ]+',
@@ -108,6 +106,13 @@ sub __validate_and_add_options {
         prompt          => '',
         undef           => '',
     };
+};
+
+
+sub __validate_and_add_options {
+    my ( $self, $opt ) = @_;
+    return if ! defined $opt;
+    my $valid = $self->__valid_options();
     my $sub =  ( caller( 1 ) )[3];
     $sub =~ s/^.+::([^:]+)\z/$1/;
     for my $key ( keys %$opt ) {
@@ -191,8 +196,13 @@ sub config {
 
 sub choose {
     if ( ref $_[0] ne 'Term::Choose' ) {
-        return Term::Choose->new()->choose( @_ );
+        return Term::Choose->new()->__choose( @_ );
     }
+    my $self = shift;
+    return $self->__choose( @_ );
+}
+
+sub __choose {
     my $self = shift;
     my ( $orig_list_ref, $opt ) = @_;
     croak "choose: called with " . @_ . " arguments - 1 or 2 arguments expected" if @_ < 1 || @_ > 2;
@@ -478,7 +488,6 @@ sub choose {
             exit 1;
         }
         elsif ( $key == KEY_ENTER ) {
-            my @chosen;
             if ( ! defined $self->{wantarray} ) {
                 $self->__reset_term( 1 );
                 return;
@@ -662,8 +671,7 @@ sub __length_longest {
         my $len = [];
         my $longest = 0;
         for my $i ( 0 .. $#$list ) {
-            my $gcs = Unicode::GCString->new( $list->[$i] );
-            $len->[$i] = $gcs->columns();
+            $len->[$i] = $self->__print_columns( $list->[$i] );
             $longest = $len->[$i] if $len->[$i] > $longest;
         }
         $self->{length_longest} = $longest;
@@ -676,8 +684,11 @@ sub __write_first_screen {
     my ( $self ) = @_;
     ( $self->{term_width}, $self->{term_height} ) = $self->{plugin}->__get_term_size();
     ( $self->{avail_width}, $self->{avail_height} ) = ( $self->{term_width}, $self->{term_height} );
-    if ( defined $self->{max_width} ) {
-        $self->{max_width} += WIDTH_CURSOR;
+    if ( $self->{length_longest} > $self->{avail_width} && $^O ne 'MSWin32' ) {
+        $self->{avail_width} += WIDTH_CURSOR;
+        # + WIDTH_CURSOR: use also the last terminal column if there is only one print-column;
+        #                 with only one print-column the output doesn't get messed up if an item
+        #                 reaches the right edge of the terminal on a non-MSWin32-OS
     }
     if ( $self->{max_width} && $self->{avail_width} > $self->{max_width} ) {
         $self->{avail_width} = $self->{max_width};
@@ -740,8 +751,7 @@ sub __prepare_promptline {
     }
     $self->{prompt} =~ s/[^\n\P{Space}]/ /g;
     $self->{prompt} =~ s/[^\n\P{C}]//g;
-    my $gcs_prompt = Unicode::GCString->new( $self->{prompt} );
-    if ( $self->{prompt} !~ /\n/ && $gcs_prompt->columns() <= $self->{avail_width} ) {
+    if ( $self->{prompt} !~ /\n/ && $self->__print_columns( $self->{prompt} ) <= $self->{avail_width} ) {
         $self->{nr_prompt_lines} = 1;
         $self->{prompt_copy} = $self->{prompt} . "\n\r";
     }
@@ -778,8 +788,7 @@ sub __size_and_layout {
         for my $idx ( 0 .. $#{$self->{list}} ) {
             $all_in_first_row .= $self->{list}[$idx];
             $all_in_first_row .= ' ' x $self->{pad_one_row} if $idx < $#{$self->{list}};
-            my $gcs_first_row = Unicode::GCString->new( $all_in_first_row );
-            if ( $gcs_first_row->columns() > $self->{avail_width} ) {
+            if ( $self->__print_columns( $all_in_first_row ) > $self->{avail_width} ) {
                 $all_in_first_row = '';
                 last;
             }
@@ -789,23 +798,12 @@ sub __size_and_layout {
         $self->{rc2idx}[0] = [ 0 .. $#{$self->{list}} ];
     }
     elsif ( $self->{layout} == 3 ) {
-        if ( $self->{length_longest} <= $self->{avail_width} ) {
-            for my $idx ( 0 .. $#{$self->{list}} ) {
-                $self->{rc2idx}[$idx][0] = $idx;
-            }
-        }
-        else {
-            for my $idx ( 0 .. $#{$self->{list}} ) {
-                my $gcs_element = Unicode::GCString->new( $self->{list}[$idx] );
-                if ( $gcs_element->columns > $self->{avail_width} ) {
-                    $self->{list}[$idx] = $self->__unicode_trim( $gcs_element, $self->{avail_width} - 3 ) . '...';
-                }
-                $self->{rc2idx}[$idx][0] = $idx;
-            }
+        for my $idx ( 0 .. $#{$self->{list}} ) {
+            $self->{rc2idx}[$idx][0] = $idx;
         }
     }
     else {
-        my $tmp_avail_width = $self->{avail_width} + $self->{pad} - WIDTH_CURSOR;
+        my $tmp_avail_width = $self->{avail_width} + $self->{pad};
         # auto_format
         if ( $self->{layout} == 1 || $self->{layout} == 2 ) {
             my $tmc = int( @{$self->{list}} / $self->{avail_height} );
@@ -857,22 +855,9 @@ sub __size_and_layout {
 }
 
 
-sub __unicode_trim {
-    my ( $self, $gcs, $len ) = @_;
-    return '' if $len <= 0; #
-    my $pos = $gcs->pos;
-    $gcs->pos( 0 );
-    my $cols = 0;
-    my $gc;
-    while ( defined( $gc = $gcs->next ) ) {
-        if ( $len < ( $cols += $gc->columns ) ) {
-            my $ret = $gcs->substr( 0, $gcs->pos - 1 );
-            $gcs->pos( $pos );
-            return $ret->as_string;
-        }
-    }
-    $gcs->pos( $pos );
-    return $gcs->as_string;
+sub __print_columns {
+    #my $self = $_[0];
+    Unicode::GCString->new( $_[1] )->columns();
 }
 
 
@@ -976,8 +961,7 @@ sub __wr_cell {
         my $lngth = 0;
         if ( $col > 0 ) {
             for my $cl ( 0 .. $col - 1 ) {
-                my $gcs_element = Unicode::GCString->new( $self->{list}[$self->{rc2idx}[$row][$cl]] );
-                $lngth += $gcs_element->columns();
+                $lngth += $self->__print_columns( $self->{list}[$self->{rc2idx}[$row][$cl]] );
                 $lngth += $self->{pad_one_row};
             }
         }
@@ -985,8 +969,7 @@ sub __wr_cell {
         $self->{plugin}->__bold_underline() if $self->{marked}[$row][$col];
         $self->{plugin}->__reverse()        if $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
         print $self->{list}[$self->{rc2idx}[$row][$col]];
-        my $gcs_element = Unicode::GCString->new( $self->{list}[$self->{rc2idx}[$row][$col]] );
-        $self->{i_col} += $gcs_element->columns();
+        $self->{i_col} += $self->__print_columns( $self->{list}[$self->{rc2idx}[$row][$col]] );
     }
     else {
         $self->__goto( $row - $self->{row_on_top}, $col * $self->{col_width} );
@@ -999,13 +982,31 @@ sub __wr_cell {
 }
 
 
+sub __unicode_trim {
+    my ( $self, $str, $len ) = @_;
+    my $gcs = Unicode::GCString->new( $str );
+    my $pos = $gcs->pos;
+    $gcs->pos( 0 );
+    my $cols = 0;
+    my $gc;
+    while ( defined( $gc = $gcs->next ) ) {
+        if ( $len < ( $cols += $gc->columns ) ) {
+            my $ret = $gcs->substr( 0, $gcs->pos - 1 );
+            $gcs->pos( $pos );
+            return $ret->as_string;
+        }
+    }
+    $gcs->pos( $pos );
+    return $gcs->as_string;
+}
+
+
 sub __unicode_sprintf {
     my ( $self, $idx ) = @_;
     my $unicode;
     my $str_length = defined $self->{length}[$idx] ? $self->{length}[$idx] : $self->{length_longest};
     if ( $str_length > $self->{avail_col_width} ) {
-        my $gcs = Unicode::GCString->new( $self->{list}[$idx] );
-        $unicode = $self->__unicode_trim( $gcs, $self->{avail_col_width} );
+        $unicode = $self->__unicode_trim( $self->{list}[$idx], $self->{avail_col_width} - 3 ) . '...';
     }
     elsif ( $str_length < $self->{avail_col_width} ) {
         if ( $self->{justify} == 0 ) {
@@ -1046,8 +1047,7 @@ sub __mouse_info_to_key {
         if ( $row == $mouse_row ) {
             my $end_last_col = 0;
             COL: for my $col ( 0 .. $#{$self->{rc2idx}[$row]} ) {
-                my $gcs_element = Unicode::GCString->new( $self->{list}[$self->{rc2idx}[$row][$col]] );
-                my $end_this_col = $end_last_col + $gcs_element->columns() + $self->{pad_one_row};
+                my $end_this_col = $end_last_col + $self->__print_columns( $self->{list}[$self->{rc2idx}[$row][$col]] ) + $self->{pad_one_row};
                 if ( $col == 0 ) {
                     $end_this_col -= int( $self->{pad_one_row} / 2 );
                 }
@@ -1124,7 +1124,7 @@ Term::Choose - Choose items from a list interactively.
 
 =head1 VERSION
 
-Version 1.200_01
+Version 1.205_01
 
 =cut
 
@@ -1552,7 +1552,7 @@ If the available height is less than I<max_height> I<max_height> is set to the a
 
 Height in this context means print rows.
 
-I<max_height> overwrites I<keep> if I<max_height> is set and less than I<keep>.
+I<max_height> overwrites I<keep> if I<max_height> is set to a value less than I<keep>.
 
 Allowed values: 1 or greater
 
@@ -1677,44 +1677,6 @@ I<undef> or an empty list in list context.
 
 Requires Perl version 5.8.3 or greater.
 
-=head2 Modules
-
-Used modules not provided as core modules:
-
-=over
-
-=item
-
-L<Text::LineFold>
-
-=item
-
-L<Unicode::GCString>
-
-=back
-
-Additionally, if the OS is MSWin32
-
-=over
-
-=item
-
-L<Win32::Console>
-
-=back
-
-is required. Else
-
-=over
-
-=item
-
-L<Term::ReadKey>
-
-=back
-
-is additionally required.
-
 =head2 Decoded strings
 
 C<choose> expects decoded strings as array elements.
